@@ -15,7 +15,7 @@ from ...exception import *
 
 class Arc(Hold):
     def __init__(self, time: int, totime: int, fromx: float, tox: float, slideeasing: Union[str, Callable, List[Callable]], fromy: float, toy: float,
-                 color: int, isskyline: bool, skynote: list = None, fx=None):
+                 color: int, isskyline: Union[bool, str], skynote: list = None, fx=None):
         super(Arc, self).__init__(time, totime, 1)
         self.fromx: float = fromx
         self.fromy: float = fromy
@@ -23,7 +23,7 @@ class Arc(Hold):
         self.toy: float = toy
         self.slideeasing: Union[str, Callable, List[Callable]] = slideeasing
         self.color: int = color
-        self.isskyline: bool = isskyline
+        self.isskyline: Union[bool, str] = isskyline
         self.skynote: list = skynote
         self.fx: str = fx
 
@@ -38,7 +38,23 @@ class Arc(Hold):
                 slicepara.append(slicetimepara[0][i] + slicepositionpara[i])
 
             notelist = [
-                Arc(time, totime, fromx, tox, 's', fromy, toy, self.color, self.isskyline)
+                Arc(
+                    time,
+                    totime,
+                    fromx,
+                    tox,
+                    's',
+                    fromy,
+                    toy,
+                    self.color,
+                    self.isskyline,
+                    None
+                    if (self.skynote is None)
+                    else list(filter(
+                        lambda t: time <= t <= totime,
+                        self.skynote
+                    ))
+                    )
                 for time, totime, fromx, tox, fromy, toy in slicepara
             ]
             notelist = NoteGroup(notelist)
@@ -55,11 +71,18 @@ class Arc(Hold):
                     'value {} for attribute "slideeasing" is not allowed to output (only {} allowed)'.format(
                         self.slideeasing, str(validstrings.slideeasinglist)
                     ))
+            def convert_isskyline(val):
+                if val == True:
+                    return 'true'
+                elif val == False:
+                    return 'false'
+                else:
+                    return val
             arcstr = 'arc({time},{totime},{fromx:.2f},{tox:.2f},{slideeasing},{fromy:.2f},{toy:.2f},{color},{fx},' \
                     '{isskyline})'.format(
                     time=int(self.time), totime=int(self.totime), fromx=self.fromx, fromy=self.fromy,
                     slideeasing=self.slideeasing, tox=self.tox, toy=self.toy, color=int(self.color),
-                    fx=self.fx if self.fx else 'none', isskyline='true' if self.isskyline else 'false'
+                    fx=self.fx if self.fx else 'none', isskyline=convert_isskyline(self.isskyline)
                     )
             skynotestr = ''
             if self.skynote is not None:
@@ -106,11 +129,11 @@ class Arc(Hold):
                 pass
             else:
                 raise AffNoteValueError(f"invalid type '{type(value).__name__}' for attribute slideeasing (excepting str, Callable or List)")
-        elif key == 'fx' and value is not None:
-            if value not in validstrings.fxlist:
-                raise AffNoteValueError('invalid value {} for attribute "fx" (only accept {})'.format(
-                    value, str(validstrings.fxlist)
-                ))
+        # elif key == 'fx' and value is not None:
+        #     if value not in validstrings.fxlist:
+        #         raise AffNoteValueError('invalid value {} for attribute "fx" (only accept {})'.format(
+        #             value, str(validstrings.fxlist)
+        #         ))
 
     def __geteasingtype(self):
         se = self.slideeasing
@@ -173,6 +196,10 @@ class Arc(Hold):
         # more magic number))
         self.fromx = 1 - self.fromx
         self.tox = 1 - self.tox
+        if self.color == 0:
+            self.color = 1
+        elif self.color == 1:
+            self.color = 0
         return self
 
     def vmirror(self):
@@ -190,6 +217,11 @@ class Arc(Hold):
         
     def align(self, bpm: float, error: int = 3, lcm = 96):
         super(Arc, self).align(bpm, error, lcm)
+        # 防止单天键出 bug
+        if self.time == self.totime:
+            if self.skynote:
+                self.totime += 1
+        
         if self.skynote:
             for each in enumerate(self.skynote):
                 self.skynote[each[0]] = time_align(self.skynote[each[0]], bpm, error, lcm)
