@@ -60,21 +60,21 @@ def loadline(notestr: str):
     if keyword == '' and paralist is not None:
         return note.Tap(
             time=int(paralist[0]),
-            lane=int(paralist[1])
+            lane=float(paralist[1])
         )
     elif keyword == 'hold':
         return note.Hold(
             time=int(paralist[0]),
             totime=int(paralist[1]),
-            lane=int(paralist[2])
+            lane=float(paralist[2])
         )
     elif keyword == 'arc':
         if paralist[9] == 'true':  # 是否为黑线
             isskyline = True
-        else:
+        elif paralist[9] == 'false':
             isskyline = False
-            if paralist[9] != 'false':
-                raise AffNoteValueError
+        else:   # 支持designant, etc.
+            isskyline = paralist[9]
         if sub_expression:  # arctap读取
             splited = sub_expression.split(',')
             skynotetimelist = [arctap[arctap.index('(') + 1:arctap.rindex(')')] for arctap in splited]
@@ -118,15 +118,13 @@ def loadline(notestr: str):
                 time=int(paralist[0]),
                 scenetype=scenetype
             )
-        elif scenetype in ['redline', 'arcahvdistort', 'arcahvdebris', 'hidegroup']:
+        else:
             return note.SceneControl(
                 time=int(paralist[0]),
                 scenetype=scenetype,
                 x=float(paralist[2]),
                 y=int(paralist[3])
             )
-        else:
-            raise AffSceneTypeError
     elif keyword == 'flick':
         return note.Flick(
             time=int(paralist[0]),
